@import url('https://fonts.googleapis.com/css2?family=Saira:ital,wght@0,100..900;1,100..900&display=swap');

:root .taipy-layout > .md-para:has(.taipy-navbar){
    display: flex;
    justify-content: flex-end;
    align-items: center;
}

/*overwrite defaults*/
:root {
    --container-max-width: 100%
}

h1, .h1{
    color: #478eff;
    font-size: 1.3rem;
    border-bottom: .5px #478eff solid;
}

.taipy-layout{
    display: grid;
    justify-items: center;
}

.css-fge750.Mui-selected{
    background-color: #478eff;
    color: white
}

.css-fge750.Mui-selected:hover{
    background-color: rgb(71 142 255 / 75%)
}

/*menu-lateral: elemento*/
.css-1st6821 {
    background-color: #cadfe7;
    border: solid;
    border-width: thin;
    border-color: #d3d7d9;
}

/*menu-lateral: icones*/
.css-6rxl0z {
    background-color: #ffffff;
    color: #478eff;
}

.css-1rom8zt{
    min-height: 36px;
    min-width: 20px;
}

/*header-bar: elemento*/
.taipy-layout.header_container.MuiBox-root {
    position: sticky;
    top: 0;
    background-color: white;
    z-index: 98;
}

.taipy-layout.header_container.MuiBox-root.css-1we2j1x {
    align-items: center;
}

/*header-bar: botões da janela*/
.taipy-part.header_top_right_corner.MuiBox-root.css-0 {
    justify-self: right;
}

/*header-bar: logo*/
.taipy-part.header_logo.MuiBox-root.css-0 {
    justify-self: left;
    font-family: 'Saira';
}


/*toggle de texto: texto*/
.css-13vtit{
    background-color: #f1f1f1;
}
.css-vnhwe0{
    font-weight: 600;

}
.css-4jd6qr{
    border: 0px;
    padding: 6px 16px;
    text-transform: none;
}
.css-4jd6qr.Mui-selected{
    color: #478eff;
    background-color: #e1edff;
}
.css-4jd6qr.Mui-selected:hover{
    background-color: #cfe2ff;
}
div.MuiToggleButtonGroup-root.css-13vtit button[aria-pressed="false"] p {
    color: gray;
}

/*toggle de texto: elem*/

div.taipy-part.header_logo.MuiBox-root.css-0 span.MuiTypography-root.MuiTypography-body1.taipy-text.css-lpnwt2{
    font-family: 'Saira';
    font-weight: 600;
    font-size: 1.45rem;
    color: #478eff;
}

.css-etkd7z {
    font-weight: bold;
}

/*border around the full page*/
html {
    border: solid;
    border-width: thin;
    border-color: #e1edff;
}

/*menu and selector: spacing and sizing*/
.css-fge750 {
    padding: 6px 16px;
}
.css-1psnylk {
    line-height: 1.15;
}
.css-cszb8q {
    max-width: 21rem;
}

/*expandable sections: expanded*/
.css-2e6htv.Mui-expanded:last-of-type {
    background-color: rgb(202 223 231 / 25%);
}

.taipy-expandable .MuiAccordionSummary-content{
    font-size: large;
}

/*expandable sections: collapsed*/
.taipy-expandable.MuiPaper-rounded{
    background-color: rgb(202 223 231 / 25%);
}

.taipy-part.narrow_element{
    width: 90%;
    padding: 0% 0% 0% 10%;
}

/*top controls*/
.taipy-layout.top_controls.MuiBox-root {
    justify-items: flex-end;
    padding-bottom: 12px;
    z-index: 99;
}

/*top controls: slider*/
.css-xbsnaj.MuiSlider-valueLabelOpen {
    color: transparent;
    background-color: transparent;
}

.css-vnhwe0 {
    color: #478eff;
}

@media (pointer: coarse) {
    .css-1753x30 {
        padding: 0px;
    }
}

.taipy-slider {
    padding-top: 6px;
}

/*top controls: dropdown*/
.css-11z0r10.css-11z0r10.css-11z0r10 {
    width: 11rem;
}

/*interactive header*/
.taipy-layout.interactive_header.MuiBox-root {
    align-items: center;
    justify-items: legacy;
    width: -webkit-fill-available;
}

/*gambiarra (remover depois)*/
.taipy-date.invisible {
    color: transparent;
}